package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import eu.jonahbauer.wizard.client.libgdx.UiskinAtlas;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.listeners.KeyboardFocusManager;
import eu.jonahbauer.wizard.client.libgdx.state.Menu;

public class MainMenuScreen extends MenuScreen {

    private TextButton buttonPlay;
    private TextButton buttonQuit;
    private TextButton buttonInstruction;

    private final ChangeListener listener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            if (actor == buttonPlay) {
                game.getClient().execute(Menu.class, Menu::showConnectScreen);
                sfxClick();
            } else if (actor == buttonQuit) {
                sfxClick();
                Gdx.app.exit();
            } else if (actor == buttonInstruction) {
                game.getClient().execute(Menu.class, Menu::showInstructionScreen);
                sfxClick();
            }
        }
    };

    public MainMenuScreen(WizardGame game) {
        super(game);
    }

    @Override
    public void show() {
        super.show();

        int width = 160, height = 224;
        int left = 384, right = 384, top = 384, bottom = 192;
        var symbols = new Image[]{
                new Image(skin.getRegion(UiskinAtlas.SYMBOL_0)),
                new Image(skin.getRegion(UiskinAtlas.SYMBOL_1)),
                new Image(skin.getRegion(UiskinAtlas.SYMBOL_2)),
                new Image(skin.getRegion(UiskinAtlas.SYMBOL_3))
        };
        symbols[0].setPosition(left, bottom);
        symbols[1].setPosition(left, WizardGame.HEIGHT - top - height);
        symbols[2].setPosition(WizardGame.WIDTH - right - width, bottom);
        symbols[3].setPosition(WizardGame.WIDTH - right - width, WizardGame.HEIGHT - top - height);
        for (var symbol : symbols) {
            symbol.setSize(width, height);
            stage.addActor(symbol);
        }

        buttonPlay = new TextButton(messages.get("menu.main.play"), skin);
        buttonPlay.setPosition((WizardGame.WIDTH - buttonPlay.getWidth()) / 2f, 170 + 504 - 125f);
        buttonPlay.addListener(listener);
        buttonInstruction = new TextButton(messages.get("menu.instruction.main"), skin);
        buttonInstruction.setPosition((WizardGame.WIDTH - buttonInstruction.getWidth()) / 2f, 170 + 494 - 189.5f - 125f + buttonInstruction.getHeight() / 2f);
        buttonInstruction.addListener(listener);
        buttonQuit = new TextButton(messages.get("menu.main.quit"), skin);
        buttonQuit.setPosition((WizardGame.WIDTH - buttonQuit.getWidth()) / 2f, 170 + 120f);
        buttonQuit.addListener(listener);

        stage.addActor(buttonPlay);
        stage.addActor(buttonInstruction);
        stage.addActor(buttonQuit);
        stage.addCaptureListener(new KeyboardFocusManager(buttonPlay, buttonQuit));

        buttonPlay.setName("button_player");
        buttonInstruction.setName("button_instruction");
        buttonQuit.setName("button_quit");
    }
}
